{% extends 'layouts/master-page.html' %}
{% block title %}CVE's linked by {{field}}{% endblock %}
{% block head %}
  <!-- css -->
  <link href="{{ url_for('static', filename='css/custom/filter.css') }}" rel="stylesheet" />

<link href="{{ url_for('static', filename='css/dataTables.bootstrap4.min.css') }}" rel="stylesheet" />

<!-- javascript -->
<script type="text/javascript" src="{{ url_for('static', filename='js/jquery.dataTables.min.js') }}"></script>
<script type="text/javascript" src="{{ url_for('static', filename='js/dataTables.bootstrap4.min.js') }}"></script>
{% endblock %}
{% block content %}
  <!-- breadcrumb -->
{% include 'subpages/breadcrumbs.html' %}
  <table class="searchTable table">
    <tr><td class="text">Max CVSS</td>   <td>{{stats['maxCVSS']}}</td>
        <td class="text">Min CVSS</td>   <td>{{stats['minCVSS']}}</td>
        <td class="text">Total Count</td><td>{{stats['count']}}</td></tr>
  </table>
{% endblock %}

{% block body_scripts %}
<script>
  $(document).ready(function() {
       $('#CVEs').DataTable({
           "order": [[ 4, "desc" ]],
           "iDisplayLength": 10,
           "language": { "processing": "<img src='{{ url_for('static', filename='img/ajaxload.gif') }}'> Loading...",
               "zeroRecords": "No records to display", searchPlaceholder: "Search...", search: ""},
       });
       $('#CVEs').removeClass('d-none');

  } );
</script>
{% endblock %}
