{% import "bootstrap/wtf.html" as wtf %}
{% extends "layouts/auth_base.html" %}
{% block title %}Login{% endblock %}

{% block body %}
	<div class="login">
		<h1>Login</h1>
		<div class="links">
			<a href="{{ url_for('auth.login') }}" class="active">Login</a>
		</div>
		<form action="" method="post">
			{{ form.hidden_tag() }}
			<label for="username">
				<i class="fas fa-user"></i>
			</label>
			{{ form.username }}
			<label for="password">
				<i class="fas fa-lock"></i>
			</label>
			{{ form.password }}
			<div class="msg">{{ msg }}</div>
			{{ form.submit() }}
		</form>
		<div>
		{% if show_oidc %}
			<div class="msg">{{ msg }}</div>
			<button onclick="oidcLogin()">Login with OIDC</button>
		{% endif %}
		</div>
	</div>
{% endblock %}
{% block body_scripts %}
<script>
	$(document).ready(function(){
		parseStatus('{{status}}')
	});
</script>
{% endblock %}