{% extends 'layouts/master-page.html' %}
{% block title %}Most recent entries{% endblock %}
{% block head %}
<!-- css -->
<link href="{{ url_for('static', filename='css/custom/filter.css') }}" rel="stylesheet" />
<link href="{{ url_for('static', filename='css/dataTables.bootstrap4.min.css') }}" rel="stylesheet" />

<!-- javascript -->
<script type="text/javascript" src="{{ url_for('static', filename='js/jquery.dataTables.min.js') }}"></script>
<script type="text/javascript" src="{{ url_for('static', filename='js/dataTables.bootstrap4.min.js') }}"></script>
<script type="text/javascript" src="{{ url_for('static', filename='js/custom/filter.js') }}"></script>
<script type="text/javascript" src="{{ url_for('static', filename='js/custom/CVEsDataTable.js') }}"></script>
{% endblock %}
{% block content %}
<!-- breadcrumb -->
{% include 'subpages/breadcrumbs.html' %}
<!-- Filter options -->
{% include 'subpages/filters.html' %}
<!-- Search results -->
{% include 'subpages/table.html' %}
{% endblock %}

{% block body_scripts %}
<script>
  $(document).ready(function() {
    loadCVEsDataTable({{pageLength}});
    check_filter_active();
    autoClearFilterError();
  });
</script>
{% endblock %}
