.searchTable{
  border: 1px solid #CCCCCC;
}
.searchTable td{
  padding: 4px;
}
.searchTable input[type="number"]{
  width:60px;
}
.searchTable button{
  float:right;
}
.text{
  width: 100px;
}

#cvssVersion {
  margin-right: 25px;
}

#filter_warning {
  color: red;
  padding: 4px;
  margin-top: 3px;
  margin-bottom: 3px;
}

.active_filter_hilight {
  font-weight: bold;
  background-color: #ffc107;;
}