{% extends 'layouts/master-page.html' %}
{% set minimal = get_WebInterface() %}
{% block title %}CVE: {{cve['id']}}{% endblock %}
{% block head %}
    <!-- css -->
    <link href="{{ url_for('static', filename='css/custom/cve.css') }}" rel="stylesheet" />
    <!-- javascript -->
    <script type="text/javascript" src="{{ url_for('static', filename='js/custom/scripts.js') }}"></script>
    {% if not minimal %}
        <script type="text/javascript" src="{{ url_for('static', filename='js/clipboard.min.js') }}"></script>
        <script type="text/javascript" src="{{ url_for('static', filename='js/custom/cve.js') }}"></script>
    {% endif %}
{% endblock %}
{% block content %}
    <!-- breadcrumb -->
    {% include 'subpages/breadcrumbs.html' %}
    <input type="hidden" id="_cveID" value="{{cve['id']}}">
    <!-- CVE -->

    {% set keytype = ['exploitability3', 'impact3', 'cvss3', 'vulnerable_configuration_cpe_2_2', 'cvss', 'capec',
'access', 'impact', 'cvssTime', 'modified', 'published', 'summary', 'vulnerable_configuration', 'references',
'_id', 'id', 'lastModified', 'ranking', 'cwe', 'vulnerable_product', 'assigner', 'products', 'vendors',
'vulnerable_configuration_stems', 'vulnerable_product_stems', 'impactScore', 'exploitabilityScore',
'impactScore3', 'exploitabilityScore3', 'status', 'cvssVector', 'cvss3Vector', 'cvssType', 'cvss3Type', 'cvssSource',
'cvss3Source', 'cvss3Time', 'cvssTime', 'cvss4', 'cvss4Source', 'cvss4Time','cvss4Type', 'cvss4Vector', 'exploitability4',
'impact4'] %}

    <div id="CVE_INFO">
        <div class="row cve_row">
            <div class="col-md-12 cve_header">CVE Details for CVE: {{ cve['id'] }}</div>
        </div>
        <div class="row cve_row">
            <div class="col-md-9 cve_no_pad">
                <div class="row cve_row h-100">
                    <div class="col-md-1 cve_no_pad">
                        <div class="HeaderParent bg1">
                            <div class="cve_row_header">
                                Summary
                            </div>
                            <div class="DivHelper"></div>
                        </div>
                    </div>
                    <div class="col-md-11 cve_row_content">
                        {{ cve['summary'] }}
                    </div>
                </div>
            </div>
            <div class="col-md-3 cve_no_pad">
                <table class="table table-hover table-bordered cve_table table-sm">
                    <thead><tr class="thead-dark"><th colspan="2">Timestamps</th></tr></thead>
                    <tr>
                        <td>Last major update</td>
                        <td>{{ cve['modified'].strftime('%d-%m-%Y - %H:%M') }}</td>
                    </tr>
                    <tr>
                        <td>Published</td>
                        <td>{{ cve['published'].strftime('%d-%m-%Y - %H:%M') }}</td>
                    </tr>
                    {% if 'lastModified' in cve%}
                        <tr>
                            <td>Last modified</td>
                            <td>{{ cve['lastModified'].strftime('%d-%m-%Y - %H:%M') }}</td>
                        </tr>
                    {% endif %}
                    {% if 'status' in cve%}
                        <tr>
                            <td>Status</td>
                            <td>{{ cve['status'] }}</td>
                        </tr>
                    {% endif %}
                    {% if 'epss' in cve%}
                        <tr>
                            <td>EPSS</td>
                            <td>{{ cve['epssMetric']['lastModified'].strftime('%d-%m-%Y - %H:%M') }}</td>
                        </tr>
                    {% endif %}
                </table>
                <table class="table table-hover table-bordered cve_table table-sm">
                    <thead><tr class="thead-dark"><th colspan="2">Status</th></tr></thead>
                    {% if 'status' in cve%}
                        <tr>
                            <td>{{ cve['status'] }}</td>
                        </tr>
                    {% endif %}
                </table>
            </div>
        </div>
        <div class="row cve_row">
            <div class="col-md-6 cve_no_pad">
                <div class="row cve_row h-100">
                    <div class="col-md-2 cve_no_pad">
                        <div class="HeaderParent bg1">
                            <div class="cve_row_header">
                                EPSS
                            </div>
                            <div class="DivHelper"></div>
                        </div>
                    </div>
                    <div class="col-md-10 cve_no_pad cve_row_content">
                        {% if 'epss' in cve %}
                            {{ cve['epss'] }}
                        {% endif %}
                    </div>
                </div>
            </div>
            <div class="col-md-6 cve_no_pad">
                <div class="row cve_row h-100">
                    <div class="col-md-2 cve_no_pad">
                        <div class="HeaderParent bg1">
                            <div class="cve_row_header">
                                EPSS %
                            </div>
                            <div class="DivHelper"></div>
                        </div>
                    </div>
                    <div class="col-md-10 cve_no_pad cve_row_content">
                        {% if 'epssMetric' in cve %}
                            {{ cve['epssMetric']['percentile'] }}
                        {% endif %}
                    </div>
                </div>
            </div>
        </div>
        <div class="row cve_row cve_table_row_uneven">
            <div class="col-md-6 cve_no_pad">
                <div class="row cve_row h-100">
                    <div class="col-md-2 cve_no_pad">
                        <div class="HeaderParent bg1">
                            <div class="cve_row_header">
                                References
                            </div>
                            <div class="DivHelper"></div>
                        </div>
                    </div>
                    <div class="col-md-10 cve_no_pad cve_row_content">
                        <a id="colf_ref" data-toggle="collapse" href="#collapseREF" aria-expanded="false" aria-controls="collapseREF">
                            <i class="fas fa-chevron-circle-down" style="float:right"></i>
                        </a>
                        <div class="collapse partial" id="collapseREF">
                            <ul>
                                {% for ref in cve['references'] %}
                                    <li> <a href="{{ ref }}" target="_blank" rel="noreferrer">{{ ref }}</a> </li>
                                {% endfor %}
                            </ul>
                        </div>
                    </div>
                </div>
            </div>
            <div class="col-md-6 cve_no_pad">
                <div class="row cve_row h-100">
                    <div class="col-md-2 cve_no_pad">
                        <div class="HeaderParent bg1">
                            <div class="cve_row_header">
                                Vulnerable Configurations
                            </div>
                            <div class="DivHelper"></div>
                        </div>
                    </div>
                    <div class="col-md-10 cve_no_pad cve_row_content">
                        <a id="colf_vulnconf" data-toggle="collapse" href="#collapseVULN" aria-expanded="false" aria-controls="collapseVULN">
                            <i class="fas fa-chevron-circle-down" style="float:right"></i>
                        </a>
                        <div class="collapse partial" id="collapseVULN">
                            <ul>
                                {% for vulconf in cve['vulnerable_configuration'] %}
                                    <li title="{{ vulconf['id'] }} matches {{ vulconf['match'] }}">
                                        <span data-toggle="collapse" data-target="#v{{ loop.index }}" > {{ vulconf['title'] }}</span>
                                        <div id="v{{ loop.index }}" class="collapse">{{ vulconf['id'] }}</div>
                                    </li>
                                {% endfor %}
                            </ul>
                        </div>
                    </div>
                </div>
            </div>
        </div>
        <div class="row cve_row">
            {% if 'capec' in cve%}
                {% if cve['cwe']|length != 0 %}
                    <div class="col-md-6 cve_no_pad">
                        <div class="row cve_row h-100">
                            <div class="col-md-2 cve_no_pad">
                                <div class="HeaderParent bg1">
                                    <div class="cve_row_header">
                                        CAPEC
                                    </div>
                                    <div class="DivHelper"></div>
                                </div>
                            </div>
                            <div class="col-md-10 cve_no_pad cve_row_content">
                                <small>Click the CAPEC title to display a description</small>
                                <a id="colf_CAPEC" data-toggle="collapse" href="#collapseCAPEC" aria-expanded="false" aria-controls="collapseCAPEC">
                                    <i class="fas fa-chevron-circle-down" style="float:right"></i>
                                </a>
                                <div class="collapse partial" id="collapseCAPEC">
                                    <ul>
                                        {% for c in cve['capec'] %}
                                            <li class="capec_list">
                                                <a href="/capec/{{c['id']}}"><i class="fas fa-info-circle"></i></a>
                                                <a class="pointer capec" data-toggle="collapse" data-target="#c{{ loop.index }}" aria-expanded="false" aria-controls="c{{ loop.index }}"> {{c['name']}} </a>
                                                <div id="c{{ loop.index }}" class="collapse capec_sum"> {{c['summary']}} </div>
                                            </li>
                                        {% endfor %}
                                    </ul>
                                </div>
                            </div>
                        </div>
                    </div>
                {% endif %}
            {% endif %}
            {% if 'cwe' in cve and cve['cwe'] %}
                {% if cve['cwe'][0] != 'Unknown' %}
                    <div class="col-md-6 cve_no_pad">
                        <div class="row cve_row h-100">
                            <div class="col-md-2 cve_no_pad">
                                <div class="HeaderParent bg1">
                                    <div class="cve_row_header">
                                        CWE
                                    </div>
                                    <div class="DivHelper"></div>
                                </div>
                            </div>
                            <div class="col-md-10 cve_no_pad cve_row_content">
                                {% for cwe in cve['cwe'] if cwe and '-' in cwe %}
                                    <a href="/cwe/{{ cwe.split('-')[1] }}" target="_blank">{{ cwe }}</a>
                                    {% if not loop.last %}, {% endif %}
                                {% endfor %}
                            </div>
                        </div>
                    </div>
                {% endif %}
        {% endif %}
        </div>
        <div class="row cve_row">
            <div class="col-md-6 cve_first_in_row">
                <div class="col-md-12 cvss_header">CVSS</div>
                <div class="row cve_row">
                    <div class="col-md-12 cve_no_pad">
                        <div class="row cve_row h-100">
                            <div class="col-md-4 cve_no_pad">
                                <div class="row cve_row h-100">
                                    <div class="col-md-6 cve_no_pad">
                                        <div class="HeaderParent bg1">
                                            <div class="cve_row_header">
                                                Base
                                            </div>
                                            <div class="DivHelper"></div>
                                        </div>
                                    </div>
                                    <div class="col-md-6 cve_row_content cve_cvss_scores">
                                        {{ cve['cvss'] }}
                                    </div>
                                </div>
                            </div>
                            <div class="col-md-4 cve_no_pad">
                                <div class="row cve_row h-100">
                                    <div class="col-md-6 cve_no_pad">
                                        <div class="HeaderParent bg1">
                                            <div class="cve_row_header">
                                                Impact
                                            </div>
                                            <div class="DivHelper"></div>
                                        </div>
                                    </div>
                                    <div class="col-md-6 cve_row_content cve_cvss_scores">
                                        {{ cve['impactScore'] }}
                                    </div>
                                </div>
                            </div>
                            <div class="col-md-4 cve_no_pad">
                                <div class="row cve_row h-100">
                                    <div class="col-md-6 cve_no_pad">
                                        <div class="HeaderParent bg1">
                                            <div class="cve_row_header">
                                                Exploitability
                                            </div>
                                            <div class="DivHelper"></div>
                                        </div>
                                    </div>
                                    <div class="col-md-6 cve_row_content cve_cvss_scores">
                                        {{ cve['exploitabilityScore'] }}
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
                <div class="row cve_row">
                    <div class="col-md-12 cve_no_pad">
                        <div class="row cve_row h-100">
                            <div class="col-md-2 cve_no_pad">
                                <div class="row cve_row h-100">
                                    <div class="col-md-12 cve_no_pad">
                                        <div class="HeaderParent bg1">
                                            <div class="cve_row_header">
                                                Access
                                            </div>
                                            <div class="DivHelper"></div>
                                        </div>
                                    </div>
                                </div>
                            </div>
                            {% if 'access' in cve %}
                                <div class="col-md-10 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-12 cve_no_pad">
                                            <table class="table table-hover table-bordered cve_table table-sm">
                                                <thead><tr class="cvss_table_header"><th>Vector</th><th>Complexity</th><th>Authentication</th></tr></thead>
                                                <tr>
                                                    <td>{{cve['access']['vector']}}</td>
                                                    <td>{{cve['access']['complexity']}}</td>
                                                    <td>{{cve['access']['authentication']}}</td>
                                                </tr>
                                            </table>
                                        </div>
                                    </div>
                                </div>
                            {% endif %}
                        </div>
                    </div>
                </div>
                <div class="row cve_row">
                    <div class="col-md-12 cve_no_pad">
                        <div class="row cve_row h-100">
                            <div class="col-md-2 cve_no_pad">
                                <div class="row cve_row h-100">
                                    <div class="col-md-12 cve_no_pad">
                                        <div class="HeaderParent bg1">
                                            <div class="cve_row_header">
                                                Impact
                                            </div>
                                            <div class="DivHelper"></div>
                                        </div>
                                    </div>
                                </div>
                            </div>
                            {% if 'impact' in cve %}
                                <div class="col-md-10 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-12 cve_no_pad">
                                            <table class="table table-hover table-bordered cve_table table-sm">
                                                <thead><tr class="cvss_table_header"><th>Confidentiality</th><th>Integrity</th><th>Availability</th></tr></thead>
                                                <tr>
                                                    <td class="impact-{{cve['impact']['confidentiality']|lower}}">{{cve['impact']['confidentiality']}}</td>
                                                    <td class="impact-{{cve['impact']['integrity']|lower}}">{{cve['impact']['integrity']}}</td>
                                                    <td class="impact-{{cve['impact']['availability']|lower}}">{{cve['impact']['availability']}}</td>
                                                </tr>
                                            </table>
                                        </div>
                                    </div>
                                </div>
                            {% endif %}
                        </div>
                    </div>
                </div>
                <div class="row cve_row">
                    <div class="col-md-12 cve_no_pad">
                        <div class="row cve_row h-100">
                            <div class="col-md-12 cve_no_pad">
                                <div class="row cve_row h-100">
                                    <div class="col-md-12 cve_no_pad">
                                        <table class="table table-hover table-bordered cve_table table-sm">
                                            <thead><tr class="cvss_table_header"><th>Source</th><th>Type</th><th>Vector</th></tr></thead>
                                            <tr>
                                                <td>{{cve['cvssSource']}}</td>
                                                <td>{{cve['cvssType']}}</td>
                                                <td>{{cve['cvssVector']}}</td>
                                            </tr>
                                        </table>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
            <div class="col-md-6 cve_last_in_row">
                <div class="col-md-12 cvss_header">CVSS3</div>
                <div class="row cve_row">
                    {% if cve['cvss3'] is not none %}
                        <div class="col-md-12 cve_no_pad">
                            <div class="row cve_row h-100">
                                <div class="col-md-4 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-6 cve_no_pad">
                                            <div class="HeaderParent bg1">
                                                <div class="cve_row_header">
                                                    Base
                                                </div>
                                                <div class="DivHelper"></div>
                                            </div>
                                        </div>
                                        <div class="col-md-6 cve_row_content cve_cvss_scores">
                                            {{ cve['cvss3'] }}
                                        </div>
                                    </div>
                                </div>
                                <div class="col-md-4 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-6 cve_no_pad">
                                            <div class="HeaderParent bg1">
                                                <div class="cve_row_header">
                                                    Impact
                                                </div>
                                                <div class="DivHelper"></div>
                                            </div>
                                        </div>
                                        <div class="col-md-6 cve_row_content cve_cvss_scores">
                                            {{ cve['impactScore3'] }}
                                        </div>
                                    </div>
                                </div>
                                <div class="col-md-4 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-6 cve_no_pad">
                                            <div class="HeaderParent bg1">
                                                <div class="cve_row_header">
                                                    Exploitability
                                                </div>
                                                <div class="DivHelper"></div>
                                            </div>
                                        </div>
                                        <div class="col-md-6 cve_row_content cve_cvss_scores">
                                            {{ cve['exploitabilityScore3'] }}
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    {% else %}
                        <div class="col-md-12 cve_no_pad">
                            <div class="row cve_row h-100">
                                <div class="col-md-12 cve_no_pad">
                                    <div class="HeaderParent bg1">
                                        <div class="cve_row_header">
                                            None
                                        </div>
                                        <div class="DivHelper"></div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    {% endif %}
                </div>
                <div class="row cve_row">
                    <div class="col-md-12 cve_no_pad">
                        <div class="row cve_row h-100">
                            {% if cve['cvss3'] is not none %}
                                <div class="col-md-2 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-12 cve_no_pad">
                                            <div class="HeaderParent bg1">
                                                <div class="cve_row_header">
                                                    Access
                                                </div>
                                                <div class="DivHelper"></div>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                                <div class="col-md-10 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-12 cve_no_pad">
                                            <table class="table table-hover table-bordered cve_table table-sm">
                                                <thead><tr class="cvss_table_header"><th>Attack Complexity</th><th>Attack vector</th><th>Privileges Required</th><th>Scope</th><th>User Interaction</th></tr></thead>
                                                <tr>
                                                    <td>{{cve['exploitability3']['attackcomplexity']}}</td>
                                                    <td>{{cve['exploitability3']['attackvector']}}</td>
                                                    <td>{{cve['exploitability3']['privilegesrequired']}}</td>
                                                    <td>{{cve['exploitability3']['scope']}}</td>
                                                    <td>{{cve['exploitability3']['userinteraction']}}</td>
                                                </tr>
                                            </table>
                                        </div>
                                    </div>
                                </div>
                            {% endif %}
                        </div>
                    </div>
                </div>
                <div class="row cve_row">
                    <div class="col-md-12 cve_no_pad">
                        <div class="row cve_row h-100">
                            {% if cve['cvss3'] is not none %}
                                <div class="col-md-2 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-12 cve_no_pad">
                                            <div class="HeaderParent bg1">
                                                <div class="cve_row_header">
                                                    Impact
                                                </div>
                                                <div class="DivHelper"></div>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                                <div class="col-md-10 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-12 cve_no_pad">
                                            <table class="table table-hover table-bordered cve_table table-sm">
                                                <thead><tr class="cvss_table_header"><th>Confidentiality</th><th>Integrity</th><th>Availability</th></tr></thead>
                                                <tr>
                                                    <td class="impact-{{cve['impact3']['confidentiality']|lower}}">{{cve['impact3']['confidentiality']}}</td>
                                                    <td class="impact-{{cve['impact3']['integrity']|lower}}">{{cve['impact3']['integrity']}}</td>
                                                    <td class="impact-{{cve['impact3']['availability']|lower}}">{{cve['impact3']['availability']}}</td>
                                                </tr>
                                            </table>
                                        </div>
                                    </div>
                                </div>
                            {% endif %}
                        </div>
                    </div>
                </div>
                <div class="row cve_row">
                    <div class="col-md-12 cve_no_pad">
                        <div class="row cve_row h-100">
                            <div class="col-md-12 cve_no_pad">
                                <div class="row cve_row h-100">
                                    <div class="col-md-12 cve_no_pad">
                                        <table class="table table-hover table-bordered cve_table table-sm">
                                            <thead><tr class="cvss_table_header"><th>Source</th><th>Type</th><th>Vector</th></tr></thead>
                                            <tr>
                                                <td>{{cve['cvss3Source']}}</td>
                                                <td>{{cve['cvss3Type']}}</td>
                                                <td>{{cve['cvss3Vector']}}</td>
                                            </tr>
                                        </table>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        <div class="col-md-6 cve_last_in_row">
                <div class="col-md-12 cvss_header">CVSS4</div>
                <div class="row cve_row">
                    {% if cve['cvss4'] is not none %}
                        <div class="col-md-12 cve_no_pad">
                            <div class="row cve_row h-100">
                                <div class="col-md-4 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-6 cve_no_pad">
                                            <div class="HeaderParent bg1">
                                                <div class="cve_row_header">
                                                    Base
                                                </div>
                                                <div class="DivHelper"></div>
                                            </div>
                                        </div>
                                        <div class="col-md-6 cve_row_content cve_cvss_scores">
                                            {{ cve['cvss4'] }}
                                        </div>
                                    </div>
                                </div>
                                <div class="col-md-4 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-6 cve_no_pad">
                                            <div class="HeaderParent bg1">
                                                <div class="cve_row_header">
                                                    Impact
                                                </div>
                                                <div class="DivHelper"></div>
                                            </div>
                                        </div>
                                        <div class="col-md-6 cve_row_content cve_cvss_scores">
                                            {{ cve['impactScore4'] }}
                                        </div>
                                    </div>
                                </div>
                                <div class="col-md-4 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-6 cve_no_pad">
                                            <div class="HeaderParent bg1">
                                                <div class="cve_row_header">
                                                    Exploitability
                                                </div>
                                                <div class="DivHelper"></div>
                                            </div>
                                        </div>
                                        <div class="col-md-6 cve_row_content cve_cvss_scores">
                                            {{ cve['exploitabilityScore4'] }}
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    {% else %}
                        <div class="col-md-12 cve_no_pad">
                            <div class="row cve_row h-100">
                                <div class="col-md-12 cve_no_pad">
                                    <div class="HeaderParent bg1">
                                        <div class="cve_row_header">
                                            None
                                        </div>
                                        <div class="DivHelper"></div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    {% endif %}
                </div>
                <div class="row cve_row">
                    <div class="col-md-12 cve_no_pad">
                        <div class="row cve_row h-100">
                            {% if cve['cvss4'] is not none %}
                                <div class="col-md-2 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-12 cve_no_pad">
                                            <div class="HeaderParent bg1">
                                                <div class="cve_row_header">
                                                    Access
                                                </div>
                                                <div class="DivHelper"></div>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                                <div class="col-md-10 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-12 cve_no_pad">
                                            <table class="table table-hover table-bordered cve_table table-sm">
                                                <thead><tr class="cvss_table_header"><th>Attack Complexity</th><th>Attack vector</th><th>Privileges Required</th><th>User Interaction</th></tr></thead>
                                                <tr>
                                                    <td>{{cve['exploitability4']['attackcomplexity']}}</td>
                                                    <td>{{cve['exploitability4']['attackvector']}}</td>
                                                    <td>{{cve['exploitability4']['privilegesrequired']}}</td>
                                                    <td>{{cve['exploitability4']['userinteraction']}}</td>
                                                </tr>
                                            </table>
                                        </div>
                                    </div>
                                </div>
                            {% endif %}
                        </div>
                    </div>
                </div>
                <div class="row cve_row">
                    <div class="col-md-12 cve_no_pad">
                        <div class="row cve_row h-100">
                            {% if cve['cvss4'] is not none %}
                                <div class="col-md-2 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-12 cve_no_pad">
                                            <div class="HeaderParent bg1">
                                                <div class="cve_row_header">
                                                    Impact vulnerable system
                                                </div>
                                                <div class="DivHelper"></div>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                                <div class="col-md-10 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-12 cve_no_pad">
                                            <table class="table table-hover table-bordered cve_table table-sm">
                                                <thead><tr class="cvss_table_header"><th>Confidentiality</th><th>Integrity</th><th>Availability</th></tr></thead>
                                                <tr>
                                                    <td class="impact-{{cve['impact4']['vulnerable_system_confidentiality']|lower}}">{{cve['impact4']['vulnerable_system_confidentiality']}}</td>
                                                    <td class="impact-{{cve['impact4']['vulnerable_system_integrity']|lower}}">{{cve['impact4']['vulnerable_system_integrity']}}</td>
                                                    <td class="impact-{{cve['impact4']['vulnerable_system_availability']|lower}}">{{cve['impact4']['vulnerable_system_availability']}}</td>
                                                </tr>
                                            </table>
                                        </div>
                                    </div>
                                </div>
                            {% endif %}
                        </div>
                    </div>
                </div>
                <div class="row cve_row">
                    <div class="col-md-12 cve_no_pad">
                        <div class="row cve_row h-100">
                            {% if cve['cvss4'] is not none %}
                                <div class="col-md-2 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-12 cve_no_pad">
                                            <div class="HeaderParent bg1">
                                                <div class="cve_row_header">
                                                    Impact subsequent system
                                                </div>
                                                <div class="DivHelper"></div>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                                <div class="col-md-10 cve_no_pad">
                                    <div class="row cve_row h-100">
                                        <div class="col-md-12 cve_no_pad">
                                            <table class="table table-hover table-bordered cve_table table-sm">
                                                <thead><tr class="cvss_table_header"><th>Confidentiality</th><th>Integrity</th><th>Availability</th></tr></thead>
                                                <tr>
                                                    <td class="impact-{{cve['impact4']['subsequent_system_confidentiality']|lower}}">{{cve['impact4']['subsequent_system_confidentiality']}}</td>
                                                    <td class="impact-{{cve['impact4']['subsequent_system_integrity']|lower}}">{{cve['impact4']['subsequent_system_integrity']}}</td>
                                                    <td class="impact-{{cve['impact4']['subsequent_system_availability']|lower}}">{{cve['impact4']['subsequent_system_availability']}}</td>
                                                </tr>
                                            </table>
                                        </div>
                                    </div>
                                </div>
                            {% endif %}
                        </div>
                    </div>
                </div>
                <div class="row cve_row">
                    <div class="col-md-12 cve_no_pad">
                        <div class="row cve_row h-100">
                            <div class="col-md-12 cve_no_pad">
                                <div class="row cve_row h-100">
                                    <div class="col-md-12 cve_no_pad">
                                        <table class="table table-hover table-bordered cve_table table-sm">
                                            <thead><tr class="cvss_table_header"><th>Source</th><th>Type</th><th>Vector</th></tr></thead>
                                            <tr>
                                                <td>{{cve['cvss4Source']}}</td>
                                                <td>{{cve['cvss4Type']}}</td>
                                                <td>{{cve['cvss4Vector']}}</td>
                                            </tr>
                                        </table>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
        <div class="row cve_row">
            <div class="col-md-12 cve_no_pad">
                <div class="col-md-12 cvss_header">VIA4 references</div>
                {% for key, value in cve|dictsort %}
                    {% if not key in keytype %}
                        <div class="row cve_row">
                            <div class="col-md-12 cve_no_pad">
                                <div class="row cve_row h-100">
                                    <div class="col-md-12 cve_no_pad">
                                        <div class="row cve_row h-100">
                                            <div class="col-md-2 cve_no_pad">
                                                <div class="HeaderParent bg1">
                                                    <div class="cve_row_header">
                                                        {{ key }}
                                                        <span class="badge badge-info"><a href="https://github.com/CVE-Search/VIA4CVE/" target="_blank">via4</a></span>
                                                    </div>
                                                    <div class="DivHelper"></div>
                                                </div>
                                            </div>
                                            <div class="col-md-10 cve_row_content">
                                                {{ JSON2HTMLTable(value, key)|safe }}
                                            </div>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    {% endif %}
                {% endfor %}
            </div>
        </div>
    </div>
{% endblock %}
