{% extends 'layouts/master-page.html' %}
{% block title %}Vendor browsing{% endblock %}
{% block head %}
  <link href="{{ url_for('static', filename='css/custom/filter.css') }}" rel="stylesheet" />

<link href="{{ url_for('static', filename='css/dataTables.bootstrap4.min.css') }}" rel="stylesheet" />

<!-- javascript -->
<script type="text/javascript" src="{{ url_for('static', filename='js/jquery.dataTables.min.js') }}"></script>
<script type="text/javascript" src="{{ url_for('static', filename='js/dataTables.bootstrap4.min.js') }}"></script>
<script type="text/javascript" src="{{ url_for('static', filename='js/custom/CVEsDataTable.js') }}"></script>
{% endblock %}
{% block content %}
  <!-- breadcrumb -->
  {% include 'subpages/breadcrumbs.html' %}
  {% if errors %}
    <div class="well">
      <p class="lead">While searching, the following error(s) occurred:</p>
      {% if 'textsearch' in errors%}
        <b>Fulltext search not enabled</b>
        <p>The current settings in the database do not allow fulltext search.<br />
           If you feel like this should be enabled, please contact your administrator.</p>
      {% endif %}
    </div>
  {% endif %}

{% include 'subpages/table.html' %}

{% endblock %}
{% block body_scripts %}
<script>
  $(document).ready(function() {
    loadSearchDataTable({{pageLength}}, '{{freetextsearch}}', '{{vendor}}', '{{product}}');
  } );
</script>
{% endblock %}
