{% set minimal = get_WebInterface() %}
<html lang="en">
  <head>
    <!-- metadata -->
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
    <meta name="author" content="https://github.com/cve-search/cve-search/ - cve-search"/>

    <!-- css -->
    <link href="{{ url_for('static', filename='css/bootstrap.min.css') }}" rel="stylesheet">
    <link href="{{ url_for('static', filename='css/style.css') }}" rel="stylesheet">
    <link href="{{ url_for('static', filename='css/all.min.css') }}" rel="stylesheet">
    <link href="{{ url_for('static', filename='css/sweetalert2/bootstrap-4.min.css') }}" rel="stylesheet" />

    <!-- favicon -->
    <link rel="shortcut icon" href="{{ url_for('static', filename='img/favicon.ico') }}" />

    <!-- javascript -->
    <script>var MountPath = "{{url_for('home.index')}}";</script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/jquery-3.7.1.min.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/popper.min.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/bootstrap.min.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/all.min.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/sweetalert2.min.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/custom/scripts.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/custom/statusses.js') }}"></script>
    <title>{% block title %}{% endblock %} - CVE-Search</title>
    {% block head %}{% endblock %}
  </head>
  <body>
    {% include 'subpages/menu.html' %}
    <div id="main" class="container-fluid content_top_margin">
      <div class="row">
        <div class="col-sm-12">
          <!-- Status window-->
          <div id="status-box"></div>
          <!-- End Status Window-->
          <!-- Body Start -->
          <div id="content">
            {% block content %}{% endblock %}
          </div>
          <a href="#" class="back-to-top"><i class="fas fa-chevron-up"></i></a>
          <!-- Body End -->
        </div>
      </div>
    </div>
    {% block body_scripts %}
    {% endblock %}
  </body>
</html>
