<!DOCTYPE html>
<html lang="en">
  <head>
    <!-- metadata -->
    <meta charset="UTF-8">
    <title>{% block title %}{% endblock %} - CVE-Search</title>
    <meta name="author" content="https://github.com/cve-search/cve-search/ - cve-search"/>

    <!-- css -->
    <link href="{{ url_for('static', filename='css/bootstrap.min.css') }}" rel="stylesheet">
    <link href="{{ url_for('static', filename='css/custom/auth.css') }}" rel="stylesheet" >
    <link href="{{ url_for('static', filename='css/style.css') }}" rel="stylesheet" >
    <link href="{{ url_for('static', filename='css/all.min.css') }}" rel="stylesheet">
    <link href="{{ url_for('static', filename='css/sweetalert2/bootstrap-4.min.css') }}" rel="stylesheet" />

    <!-- favicon -->
    <link rel="shortcut icon" href="{{ url_for('static', filename='/img/favicon.ico') }}" />

    <!-- javascript -->
    <script type="text/javascript" src="{{ url_for('static', filename='js/jquery-3.7.1.min.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/popper.min.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/bootstrap.min.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/all.min.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/sweetalert2.min.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/custom/statusses.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/custom/oidcLogin.js') }}"></script>
  </head>
  <body>
    {% include 'subpages/menu.html' %}
      <div class="wrapper">
        {% block body %}
        {% endblock %}
        <div class="push"></div>
      </div>
    {% block body_scripts %}
    {% endblock %}
  </body>
</html>