.card {
    margin-top: 25px;
}

.margin-left {
    margin-left: 20px;
}

.margin-bottom {
    margin-bottom: 20px;
}

.margin-left-null {
    margin-left: 0px;
}

.admin_row {
	background-color: aliceblue;
	height: 100%;
	padding-left: 25px;
	padding-right: 25px;
	padding-bottom: 25px;
	margin-left: 0px;
	margin-right: 0px;
}

th[colspan="3"] {
    text-align: center;
}

.center_header {
    text-align: center;
}

.form_header {
    margin-top: 10px;
    margin-bottom: 25px;
}

/* Prevent table overflow by making the Database info card 50% wide 
   between 900px and 1300px and 100% wide below 900px. */
.row.admin_row .col-md-4 .card .table-responsive {
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
}
.row.admin_row .col-md-4 .card .admin_table {
    width: 100% !important;
    table-layout: auto !important;
}
.row.admin_row .col-md-4 .card .admin_table th,
.row.admin_row .col-md-4 .card .admin_table td {
    white-space: normal;
    word-break: break-word;
}
@media (max-width: 1300px) and (min-width: 900px) {
    .row.admin_row .col-md-4 {
        flex: 0 0 50% !important;
        max-width: 50% !important;
        width: 50% !important;
    }
}
@media (max-width: 900px) and (min-width: 768px) {
    .row.admin_row .col-md-4 {
        flex: 0 0 100% !important;
        max-width: 100% !important;
        width: 100% !important;
    }
}
