{% extends 'layouts/master-page.html' %}
{% block title %}Bookmarked CVEs{% endblock %}
{% block content %}
  <!-- breadcrumb -->
{% include 'subpages/breadcrumbs.html' %}
{% endblock %}
{% block body_scripts %}
<script>
  $(document).ready(function() {
       $('#CVEs').DataTable({
           "order": [[ 4, "desc" ]],
           "iDisplayLength": 10,
           "language": { "processing": "<img src='{{ url_for('static', filename='img/ajaxload.gif') }}'> Loading...",
               "zeroRecords": "No records to display", searchPlaceholder: "Search...", search: ""},
       });
       $('#CVEs').removeClass('d-none');

  } );
</script>
{% endblock %}
