<table id="CVEs" class="table table-hover table-striped table-bordered d-none">
  <thead class="thead-dark">
    <tr>
      <th width="2%"></th>
      <th width="6%">ID</th>
      <th width="3%">CVSS</th>
      <th width="3%">CVSS3</th>
      <th width="3%">CVSS4</th>
      <th width="60%">Summary</th>
      <th width="11%">Last (major) update</th>
      <th width="11%">Published</th>
    </tr>
  </thead>
  <tbody>

  </tbody>
</table>
<a href="#" class="select-submit">Mark selected</a>
