a {
    color: #457abf;
}
.table.table-striped tr.selected{
    background-color: #556677;
    color: #FFFFFF;
}
.listInput{
    width:calc(50% - 25px);
    min-width:200px;
}
.listInput textarea, input[type=text]{
    width:100%;
}
.listInput input[type=submit]{
    margin:0px auto;
    width:150px;
}
.listInputCenter p{
    padding-top:100px;
    text-align:center;
}
.invisiTable{
    margin: 0px -5px;
}
.invisiTable td{
    padding: 0px 5px;
    vertical-align: top;
    border: 1px dotted #dee2e6;
    line-height: 1.25;
}
a .fa-link{
    display: none;
}
.badge a{
    color:white;
}
#admin{
    float: right;
}
ul.nav li.dropdown:hover > ul.dropdown-menu {
    display: block;
}
.back-to-top {
    position: fixed;
    bottom: 2em;
    right: 0px;
    text-decoration: none;
    color: #000000;
    background-color: rgba(235, 180, 180, 0.70);
    font-size: 12px;
    padding: 1em;
    border-radius: 5px;
    display: none;
}
.select-submit {
    position: fixed;
    bottom: 6em;
    right: 0px;
    text-decoration: none;
    color: #000000;
    background-color: rgba(180, 180, 235, 0.70);
    font-size: 12px;
    padding: 1em;
    display: none;
}

.back-to-top:hover {    
    background-color: rgba(135, 135, 135, 0.50);
}

.table-nonfluid {
   width: auto;
}

.block {
   display: inline-block;
}
.semiCollapsed {
   overflow: hidden;
   text-overflow: ellipsis;
   max-height:110px;
}

.pagerdiv {
   width:100%;
   text-align:center;
}

#status {
   position:fixed;
   top: 50%;
   left: 50%;
   width:50em;
   height:6em;
   margin-top: -3em;
   margin-left: -25em;
   border: 1px solid #ccc;
   text-align:center;
}

.hidden {
   visibility:hide;
}

ul.via4 {
   padding-left:0px;
}

ul.via4 > li {
  margin: 3px 0px;
  padding-left: 3px;
  list-style-type: none;
  position: relative;
}

ul.via4 > li:after {
  padding:0;margin:0;display:block;/* not really needed */
  content: "";
  height:90%;
  width:3px;
  background-color: #8AE790;
  position: absolute;
  left:0;
  top:5%;
}

.admin_table {
    font-size: 14px;
    margin-top: 15px;
    margin-bottom: 15px;
}

.passwd_td {
    padding-right: 15px;
}

.margin_left {
    margin-left: 25px;
}

.capec_table_description {
    padding-left: 50px;
}

.capec_discription_tr {
    padding-top: 15px;
}

.typical_severity {
    font-size: 18px;
    font-weight: bold;
}

.very_low {
    color: #0062fb
}

.Low {
    color: #166201
}

.Medium {
    color: #fb8100;
}

.High {
    color: #fb0000;
}

.critical {
    color: #ad00fb !important;
}

.version {
    font-size: 12px;
    font-weight: bold;
    font-style: italic;
}

.show_filter {
    margin-bottom: 25px;
}

.content_top_margin {
    margin-top: 100px;
    width: 97%;
}

.btn-group-xs > .btn, .btn-xs {
    padding  : .25rem .4rem;
    font-size  : .875rem;
    line-height  : .5;
    border-radius : .2rem;
}

.button_filter {
    margin-left: 15px;
}

#search {
    margin-left: 25px;
}

#CVEs.table-striped tbody tr:nth-of-type(2n+1) {
	background-color: rgba(74, 70, 70, 0.14);
}

#CVEs.table-hover> tbody> tr:hover{
    background-color: rgba(240, 0, 0, 0.04);
}

.pagination {
    font-size: 12px;
    padding-bottom: 25px;
}

ul.pagination {
    font-style: italic;
}

.page-item.active .page-link {
	background-color: #6c757d;
	border-color: #6c757d;
}

a.page-link {
    color: #6c757d;
}

a.page-link:hover {
    color: white;
    background-color: #bdc3c6;
}

.dataTables_info {
    font-size: 12px;
}

#startDate, #endDate, #timeSelect, #timeTypeSelect{
    width: 250px;
    margin-right: 25px;
	flex: 0 1 auto;
	margin-top: 20px;
	margin-bottom: 20px;
}

#cvssSelect, #cvss, #rejectedSelect {
    width: 250px;
    margin-right: 25px;
	flex: 0 1 auto;
}

#labelTime {
    margin-top: 20px;
	margin-bottom: 20px;
}

#labelCVSS, #labelRejected {
    margin-bottom: 20px;
}