.cve-info {
   width: 400px;
}
.cve-info td {
   text-align: center;
   background-color: white;
}

.cve-info th {
    text-align: center;
}

.table-even {
   table-layout: fixed
}

.impact-none {
   color:green;
   font-weight: bold;
}

.impact-partial {
   color:orange;
   font-weight: bold;
}

.impact-complete {
   color:red;
   font-weight: bold;
}

.impact-high {
    color: red;
    font-weight: bold;
}

.impact-partial {
    color : orange;
    font-weight: bold;
}

.impact-low {
    color : green;
    font-weight: bold;
}

#actions {
    float:right;
}

#actions button {
    width:100%;
}

td > div {
    padding-right: 10px;
}

pre {
   white-space: pre-wrap;       /* CSS 3 */
   white-space: -moz-pre-wrap;  /* Mozilla, since 1999 */
   white-space: -pre-wrap;      /* Opera 4-6 */
   white-space: -o-pre-wrap;    /* Opera 7 */
   word-wrap: break-word;       /* Internet Explorer 5.5+ */
}

th[colspan="2"] {
    text-align: center;
}


table.table_outer > tbody > tr > td:first-child {
	background-color: #990808;
	color: white;
	font-weight: bold;
	text-align: center;
	vertical-align: middle;
	width: 250px;
}

.partial.collapse:not(.show) {
  display: block;
  /* height = lineheight * no of lines to display */
  height: 3em;
  overflow: hidden;
}

.partial.collapsing {
    height: 3em;
}

.pointer {
    cursor: pointer;
}

.pointer:hover {
    color: grey;
}

.capec {
    color: black;
}

.capec_list {
    margin-top: 10px;
    margin-bottom: 10px;
}

.capec_sum {
    font-style: italic;
    margin: 10px 0px 10px 20px;
}

.collapse > ul > li {
    margin-bottom: 5px;
}

.collapsing > ul > li {
    margin-bottom: 5px;
}

.cve_row {
    margin-left: 0px;
    margin-right: 0px;
}

.cve_last_in_row {
    padding-right: 0px;
}

.cve_first_in_row {
    padding-left: 0px;
}

.cve_no_pad {
    padding-right: 0px;
    padding-left: 0px;
}

.cve_header {
    text-align: center;
    color: white;
    padding-top: 10px;
    padding-bottom: 10px;
    font-size: 18px;
    font-weight: bold;
    background-color: #343a40;
}

.cvss_header {
    text-align: center;
    color: white;
    padding-top: 3px;
    padding-bottom: 3px;
    font-size: 18px;
    font-weight: bold;
    background-color: #73808c;
    margin-top: 15px;
}

.cve_row_header {
    color: white;
	font-weight: bold;
	text-align: center;
    display: inline-block;
    vertical-align: middle;
    white-space: normal;
    width: 100%;
    max-width: 100%;
}

.cve_row_content {
    border-style: solid;
    border-color: #dee2e6;
    border-width: 1px;
    padding: 10px;
}

.cve_table {
    margin-bottom: 0px;
    width: 100%;
    text-align: center;
}

#CVE_INFO {
    margin-bottom: 25px;
}

.HeaderParent {
    height: 100%;
    border-style: solid;
    border-color: #dee2e6;
    border-width: 1px;
    white-space: nowrap;
    width: 100%;
}

.DivHelper {
    display: inline-block;
    vertical-align: middle;
    height:100%;
}

.bg1 {
    background-color: #990808;
}

.cve_table_row_uneven {
    background-color: #f2f2f2;
}

.cve_cvss_scores {
    font-weight: bold;
    text-align: center;
}

.cve_no_data {
    font-weight: bold;
    text-align: center;
    font-size: 18px;
}

.table .cvss_table_header th {
	color: #fff;
	background-color: #c7ccd1;
	border-color: #f2f2f2;
}

.table td {
        word-wrap: break-word;
        word-break: break-all;
        white-space: normal;
}
